#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for H/XR/XV Migration."""

import os
import re

from iobase.xmldata import (XmlConfNode, XmlConf, ROOT_PATH, ANY_CHILD_PATH)
from iobase.exception import (InvalidArgError, IoBaseException)
from ionas.request import Entry

import pal_info
import libfal.falcmd as falcmd
import libfal.util as util
from fal_info import (ReqTypes, Results)

from pkgaddon.fal_hmigration_config import (CONFIG_FILE,
                                            LANDISK_INFO_XML_PATH,
                                            IGNORED_SHARE_LIST,)


TRUE_TEXT = XmlConfNode.TRUE_TEXT
FALSE_TEXT = XmlConfNode.FALSE_TEXT
MIGRATION_NAME = "migration"
JOB_NAME = "job"
LANDISKINFO_NAME = "landiskinfo"
SOURCE_PATH = "job/sourcelist/source"
LANDISKINFO_HOST_NAME_PATH = "sysinfo/hostdesc/name"
LANDISKINFO_PRODUCT_NAME_PATH = "product/name"
LANDISKINFO_PROTOCOL_PATH = "network.v2/netiflist/netif/protocollist/protocol"
DHCP_NAME = "dhcp"
ADDR_PATH = "addrlist/addr"
RESTOREUI_NAME = "restoreui"
HOSTNAMELIST_NAME = "hostnamelist"
HOSTNAME_NAME = "hostname"
VERSION_ATTR = "version"
FAMILY_ATTR = "family"
INET_VALUE = "inet"
FAMILYCONFIG_PATH = "network.v2/familyconfig"
NETIFLIST_PATH = "network.v2/netiflist"
FAMILY_INET4 = "inet4"
OLD_TCPIP_PATH = "network/tcpip"
OLD_INET6_TCPIP_PATH = """network/tcpip[@family="inet6"]"""
OLD_INET6_COMMON_PATH = OLD_INET6_TCPIP_PATH + "/common"
OLD_INET4_TCPIP_PATH = """network/tcpip[@family="inet"]"""
OLD_INET4_COMMON_PATH = OLD_INET4_TCPIP_PATH + "/common"
TYPE_ATTR = "type"
CUSTOM_VALUE = "custom"
STATIC_VALUE = "static"
DEFAULT_COMMON_TYPE = "lan"
DNSSRV_NAME = "dnssrv"
GATEWAY_NAME = "gateway"
OLD_INET6_LAN_PATH = OLD_INET6_TCPIP_PATH + "/lan"
OLD_INET6_EXT_PATH = OLD_INET6_TCPIP_PATH + "/ext"
OLD_INET4_LAN_PATH = OLD_INET4_TCPIP_PATH + "/lan"
OLD_INET4_EXT_PATH = OLD_INET4_TCPIP_PATH + "/ext"
PROTOCOLLIST_NAME = "protocollist"
IP_PATH = "ipsetting/ip"
SUBNET_PATH = "ipsetting/subnet"
PREFIXLEN_PATH = "ipsetting/prefixlen"
NETMASK_ATTR = "netmask"
PREFIXLEN_ATTR = "prefixlen"
FTP_CONTROL_NUMBER_XPATH = """ftp/port[@type="control"]/number"""
OLD_FTP_CONTROL_NUMBER_XPATH = """network/ftp/port[@type="control"]/number"""
FTP_TRANSFER_NUMBER_XPATH = """ftp/port[@type="transfer"]/number"""
OLD_FTP_TRANSFER_NUMBER_XPATH = """network/ftp/port[@type="transfer"]/number"""
FTP_ENCODE_PATH = "ftp/encode"
OLD_KEYSERVER_PATH = "volume/keyserver"
OLD_SERVER_PATH = "volume/keyserver/server"
KEYSERVERLIST_NAME = "keyserverlist"
HOST_NAME = "host"
ASSIGN_PATH = "msnetwork/assign"
OLD_MSNETWORK_PATH = "network/msnetwork"
ASSIGN_CHILD_NAME_LIST = ["wg", "nt", "ad"]
MAILCONFLIST_PATH = "notice/mailnotice/mailconflist"
MAILBASIC_MAILCONF_PATH = "notice/mailbasic/mailconf"
OLD_REPORT_DISKUSE_PATH = "system/report/diskuse"
OLD_EVENTNOTICE_CHILD_PATH = "system/eventnotice/*"
OLD_MAILTOINFO_PATH = "system/mail/mailtolist/mailtoinfo"
MAILTO_NAME = "mailto"
ENCODE_NAME = "encode"
DETAIL_ATTR = "detail"
REGULARNOTICE_PATH = "notice/regularnotice"
REPORT_PATH = "notice/report"
EVENTTIMELIST_NAME = "eventtimelist"
OLD_REPORT_DAY_PATH = "system/report/diskuse/day"
USERLIST_PATH = "users/userlist"
OLD_USER_PATH = "userlist/user"
GROUPLIST_PATH = "groups/grouplist"
OLD_GROUP_PATH = "grouplist/group"
USERLIST_NAME = "userlist"
SHARELIST_PATH = "shares/sharelist"
SHARE_NAME = "share"
NAME_NAME = "name"
UNDELETABLE_ATTR = "undeletable"
OLD_FOLDER_PATH = "share/folder"
VOLUME_NAME = "volume"
SHARETARGET_NAME = "sharetarget"
PERMISSION_NAME = "permission"
DETAILPERMISSION_PATH = "permission/detailpermission"
POWERSETTING_PATH = "powersetting"
OLD_POWERSCHEDULE_PATH = "system/powersetting/powerschedule"
NO_TAG_EXISTS_NAME = "no_tag_exists"
USBLIST_PATH = "portconfig/usblist"
USB_NAME = "usb"
PORT_ATTR = "port"
PORTCONFIG_TARGET_PORT = "1"
OLD_USB1_PATH = """volume/usb[@port="1"]"""
MODE_ATTR = "mode"
QUICKCOPY_NAME = "quickcopy"
USB1_MODE_PATH = """portconfig/usblist/usb[@port="1"]/modelist/mode"""
SHARE_ATTR = "share"

STATUSINFO_PATH = "statusinfo"
HOST_XML_TMPL = "<host><name>%s</name></host>"
PRODUCT_XML_TMPL = "<product><name>%s</name></product>"
INET6_XML_TMPL = """<inet6>
        <dns source="%s">
            <addrlist>
                <addr>%s</addr>
            </addrlist>
        </dns>
        <gateway source="%s">
            <addr>%s</addr>
        </gateway>
</inet6>
"""
INET4_XML_TMPL = """<inet4>
        <dns source="%s">
            <addrlist>
                <addr>%s</addr>
            </addrlist>
        </dns>
        <gateway source="%s">
            <addr>%s</addr>
        </gateway>
</inet4>
"""

MAIN_FAMILY = "inet4"
NETWORK_COMMON_TYPE_MAP = {
        "lan": "lan1",
        "ext": "lan2",
        CUSTOM_VALUE: STATIC_VALUE,
    }

NETIF_OLD_XPATH_MAP = {
        "lan1": {
                "inet6": OLD_INET6_LAN_PATH,
                "inet4": OLD_INET4_LAN_PATH,
            },
        "lan2": {
                "inet6": OLD_INET6_EXT_PATH,
                "inet4": OLD_INET4_EXT_PATH,
            },
    }

FTP_ENCODE_MAP = {
        "utf8": "utf-8",
    }

DEFAULT_VER_STR = "1.00"
HDL_H_MIN_VER = 3.00
MAX_MAILTO_NUM = 5

NEW_LANDISKINFO_BASE_TMPL = \
"""<landiskinfo update="%(update)s" version="%(version)s">
    <product>
        <name>%(product_name)s</name>
        <firmware>%(firmware)s</firmware>
    </product>
    <network/>
    <network.v2>
        <familyconfig/>
        <netiflist/>
    </network.v2>
    <applenetwork>
        <service enable="%(applenetwork_enable)s"/>
    </applenetwork>
    <backups>
        <backup>
            <joblist/>
        </backup>
        <datacopy>
            <joblist/>
        </datacopy>
    </backups>
    <eco>
        <time>
            <minute>%(eco_time)s</minute>
        </time>
        <volumelist>
            <volume enable="%(eco_sataraid1_enable)s">
                <name>sataraid1</name>
            </volume>
            <volume enable="%(eco_usb1_enable)s">
                <name>usb1</name>
            </volume>
            <volume enable="%(eco_usb2_enable)s">
                <name>usb2</name>
            </volume>
        </volumelist>
    </eco>
    <firmware>
        <updatenotice enable="%(fw_updatenotice_enable)s"/>
        <config/>
    </firmware>
    <ftp>
        <service enable="%(ftp_enable)s"/>
        <port type="control">
            <number/>
        </port>
        <port type="transfer">
            <number/>
        </port>
        <encode>%(ftp_encode)s</encode>
    </ftp>
    <groups>
        <grouplist/>
    </groups>
    <led>
        <brightness>%(led)s</brightness>
    </led>
    <msnetwork>
        <service enable="%(msnetwork_enable)s"/>
        <assign method="%(msnetwork_method)s"/>
        <winssrv>%(winssrv)s</winssrv>
    </msnetwork>
    <notice>
        <mailbasic>
            <mailconf>
                <host type="smtp">%(mailconf_host)s</host>
                <port type="smtp">%(mailconf_port)s</port>
                <fromaddr>%(mailconf_fromaddr)s</fromaddr>
                <auth type="%(mailconf_auth_type)s">
                    <user>%(mailconf_auth_user)s</user>
                    <password>%(mailconf_auth_password)s</password>
                    <pop3>
                        <host>%(mailconf_pop_host)s</host>
                        <port>110</port>
                    </pop3>
                </auth>
                <encode>%(mailconf_encode)s</encode>
            </mailconf>
        </mailbasic>
        <mailnotice>
            <mailconflist/>
        </mailnotice>
        <report>
            <eventtimelist/>
        </report>
        <regularnotice enable="false">
            <diskerror enable="%(notice_diskerror_enable)s"/>
            <ups enable="%(notice_ups_enable)s"/>
            <volume enable="%(notice_volume_enable)s">%(notice_volume_value)s</volume>
            <quota enable="%(notice_quota_enable)s">%(notice_quota_value)s</quota>
        </regularnotice>
    </notice>
    <portconfig>
        <usblist>
            <usb port="1">
                <modelist>
                    <mode enable="false">share</mode>
                    <mode enable="true" share="disk1">quickcopy</mode>
                </modelist>
            </usb>
        </usblist>
    </portconfig>
    <powersetting>
        <autoresume enable="%(autoresume_enable)s"/>
    </powersetting>
    <quota>
        <volumelist>
            <volume enable="%(quota_enable)s">/mnt/sataraid1</volume>
        </volumelist>
    </quota>
    <raidctrl>
        <activerepair>
            <schedule enable="%(activerepair_enable)s">
                <day>%(activerepair_day)s</day>
                <time>
                    <hour>%(activerepair_hour)s</hour>
                    <minute>%(activerepair_minute)s</minute>
                </time>
            </schedule>
        </activerepair>
        <raidlist/>
    </raidctrl>
    <shares>
        <sharelist/>
    </shares>
    <switch>
        <reset enable="%(reset_enable)s"/>
        <func enable="%(func_enable)s"/>
    </switch>
    <sysinfo>
        <hostdesc>
            <name>%(host_name)s</name>
            <location/>
            <desc>%(host_desc)s</desc>
        </hostdesc>
    </sysinfo>
    <systemlog>
        <accesslog enable="%(accesslog_enable)s"/>
    </systemlog>
    <ups>
        <netshutdown>
            <targetlandisklist>
                <targetlandisk number="1" enable="%(ups_target1_enable)s">
                    <host>%(ups_target1_host)s</host>
                    <passwd>%(ups_target1_passwd)s</passwd>
                </targetlandisk>
                <targetlandisk number="2" enable="%(ups_target2_enable)s">
                    <host>%(ups_target2_host)s</host>
                    <passwd>%(ups_target2_passwd)s</passwd>
                </targetlandisk>
                <targetlandisk number="3" enable="%(ups_target3_enable)s">
                    <host>%(ups_target3_host)s</host>
                    <passwd>%(ups_target3_passwd)s</passwd>
                </targetlandisk>
                <targetlandisk number="4" enable="%(ups_target4_enable)s">
                    <host>%(ups_target4_host)s</host>
                    <passwd>%(ups_target4_passwd)s</passwd>
                </targetlandisk>
                <targetlandisk number="5" enable="%(ups_target5_enable)s">
                    <host>%(ups_target5_host)s</host>
                    <passwd>%(ups_target5_passwd)s</passwd>
                </targetlandisk>
            </targetlandisklist>
        </netshutdown>
        <warning enable="%(ups_warning_enable)s"/>
        <time>
            <minute>%(ups_time)s</minute>
        </time>
    </ups>
    <users>
        <userlist/>
    </users>
</landiskinfo>
"""

NEW_NETIF_BASE_TMPL = """<netif name="%(name)s" enable="%(netif_enable)s" wol="%(wol_enable)s" mtu="%(mtu)s">
    <protocollist/>
</netif>
"""

NEW_PROTOCOL_BASE_TMPL = \
"""<protocol family="%(family)s" enable="%(protocol_enable)s">
    <dhcp enable="%(dhcp_enable)s"/>
    <addrlist>
        <addr/>
    </addrlist>
</protocol>
"""

LOCKKEY_BASE_XML = """<lockkey>
    <keyserverlist/>
</lockkey>
"""

NEW_KEYSERVER_BASE_TMPL = """<keyserver enable="%(enable)s">
    <registno>%(no)s</registno>
    <hostname>%(hostname)s</hostname>
    <port>%(port)s</port>
</keyserver>
"""

NEW_EVENTTIME_BASE_TMPL = """<eventtime>
    <day>%(day)s</day>
    <hour>%(hour)s</hour>
    <minute>%(minute)s</minute>
</eventtime>
"""

NEW_MAILCONF_BASE_TMPL = """<mailconf detail="false" enable="true">
    <usessl enable="false"/>
    <host type="smtp"/>
    <port type="smtp">25</port>
    <fromaddr/>
    <toaddrlist>
        <toaddr>%(toaddr)s</toaddr>
    </toaddrlist>
    <auth type="no">
       <pop3>
           <host/>
           <port>110</port>
       </pop3>
       <user/>
       <password/>
    </auth>
    <encode>iso-2022-jp</encode>
    <notifylist>
         <notify name="system" enable="%(system_enable)s"/>
         <notify name="backup" enable="%(backup_enable)s"/>
         <notify name="eventlog" enable="%(eventlog_enable)s"/>
         <notify name="accesslog" enable="%(accesslog_enable)s"/>
         <notify name="temperature" enable="%(temperature_enable)s"/>
         <notify name="disk" enable="%(disk_enable)s"/>
         <notify name="volume" enable="%(volume_enable)s"/>
         <notify name="quota" enable="%(quota_enable)s"/>
         <notify name="ups" enable="%(ups_enable)s"/>
         <notify name="report" enable="%(report_enable)s"/>
    </notifylist>
</mailconf>
"""

NEW_USER_BASE_TMPL = """<user admin="false" hidden="false" reserved="false">
    <name>%(name)s</name>
    <passwd>%(passwd)s</passwd>
    <fullname>%(fullname)s</fullname>
    <desc>%(desc)s</desc>
    <quota>
        <limit>%(quota_limit)s</limit>
    </quota>
</user>
"""

NEW_GROUP_BASE_TMPL = """<group hidden="false" reserved="false">
    <name>%(name)s</name>
    <userlist/>
    <quota>
        <limit>%(quota_limit)s</limit>
    </quota>
</group>
"""

NEW_SHARE_BASE_TMPL = """<share>
    <name>%(name)s</name>
    <volume/>
    <path>%(path)s</path>
    <desc>%(desc)s</desc>
    <permission>
        <readonly enable="%(readonly_enable)s"/>
    </permission>
    <shareservice>
        <msshare enable="%(msshare_enable)s">
            <trash enable="%(trash_enable)s"/>
            <hidden enable="%(hidden_enable)s"/>
            <quota>
                <limit>%(quota_limit)s</limit>
            </quota>
        </msshare>
        <noguest enable="%(noguest_enable)s"/>
        <ftp enable="%(ftp_enable)s"/>
        <appleshare enable="%(appleshare_enable)s" timemachine="%(timemachine_enable)s"/>
    </shareservice>
</share>
"""


class RequestError(IoBaseException):
    """Request error class."""

    pass


class FalMigration(falcmd.FalCmd):
    """FAL Migration command implementation class."""

    #
    # Internal methods
    #

    def _check_body_node(self, node, type=None):
        """Check if entry's body node is valid."""

        if node.get_node_name() != MIGRATION_NAME:
            raise InvalidArgError(node.get_node_name())

        if type == ReqTypes.APPLY:
            try:
                node.get_node(JOB_NAME)
            except:
                raise InvalidArgError("job not found")

    def _get_landiskinfo_version(self, landiskinfo):
        """Return the version of "landiskinfo"."""

        ver_str = landiskinfo.get_attr_value(VERSION_ATTR, DEFAULT_VER_STR)
        return float(ver_str)

    def _get_volume(self, sharetarget):
        """Return volmue corresponding to "sharetarget"."""

        match = re.match("/mnt/([^/]+).*", sharetarget)
        if match == None:
            return ""
        return match.group(1)

    def _transform_xml_data(self, src_xml, new_xml_base_tmpl, map_info_list):
        """Transform old share setting to new format in "landiskinfo"."""

        str_fmt_map = {}
        for (fmt_key, target_xpath, default_value, to_lower) in map_info_list:
            str_fmt_map[fmt_key] = \
                    src_xml.get_value(target_xpath, default_value)
            if to_lower:
                str_fmt_map[fmt_key] = str_fmt_map[fmt_key].lower()

        return XmlConfNode(new_xml_base_tmpl % str_fmt_map)

    def _update_old_tcpip(self, old_landiskinfo):
        """Update old "tcpip" setting in "old_landiskinfo"."""

        old_tcpip_list = old_landiskinfo.get_node_list(OLD_TCPIP_PATH)
        if len(old_tcpip_list) != 1:
            return

        tcpip_attrs = old_tcpip_list[0].get_attrs()
        if not FAMILY_ATTR in tcpip_attrs:
            old_tcpip_list[0].set_attrs({FAMILY_ATTR: INET_VALUE})

    def _update_familyconfig(self, old_landiskinfo, new_landiskinfo):
        """Update "familyconfig" of network setting in "new_landiskinfo"
           by setting in "old_landiskinfo".
        """

        familyconfig = new_landiskinfo.get_node(FAMILYCONFIG_PATH)
        for (new_inet_xml_tmpl, old_common_path) in \
                (
                    (INET6_XML_TMPL, OLD_INET6_COMMON_PATH),
                    (INET4_XML_TMPL, OLD_INET4_COMMON_PATH),
                ):

            try:
                old_common = old_landiskinfo.get_node(old_common_path)
            except:
                # Target family not supported
                continue

            common_type = old_common.get_attr_value(TYPE_ATTR).lower()
            dns_addr = ""
            gateway_addr = ""
            if common_type == CUSTOM_VALUE:
                dns_addr = old_common.get_value(DNSSRV_NAME)
                gateway_addr = old_common.get_value(GATEWAY_NAME)

            new_inet_xml = XmlConfNode(new_inet_xml_tmpl %
                                    (
                                        NETWORK_COMMON_TYPE_MAP[common_type],
                                        dns_addr,
                                        NETWORK_COMMON_TYPE_MAP[common_type],
                                        gateway_addr,
                                    ))
            familyconfig.append_child(new_inet_xml)

    def _get_netif(self, old_landiskinfo, xpath_map, map_info_list):
        """Create new "netif" for network setting by setting in
           "old_landiskinfo" and return it.
        """

        netif = self._transform_xml_data(
                            old_landiskinfo.get_node(xpath_map[MAIN_FAMILY]),
                            NEW_NETIF_BASE_TMPL,
                            map_info_list)
        protocollist = netif.get_node(PROTOCOLLIST_NAME)
        for family in sorted(xpath_map.keys()):
            PROTOCOL_BASE_MAP_INFO_LIST = [
                ["family", NO_TAG_EXISTS_NAME, family, False],
                ["protocol_enable", "@enable", TRUE_TEXT, False],
                ["dhcp_enable", "@dhcp", TRUE_TEXT, False],
            ]

            try:
                protocol = self._transform_xml_data(
                                old_landiskinfo.get_node(xpath_map[family]),
                                NEW_PROTOCOL_BASE_TMPL,
                                PROTOCOL_BASE_MAP_INFO_LIST)
                if (protocol.is_enabled() and
                    not protocol.get_node(DHCP_NAME).is_enabled()):

                    addr = protocol.get_node(ADDR_PATH)
                    old_net_dev = old_landiskinfo.get_node(xpath_map[family])
                    addr.set_text(old_net_dev.get_value(IP_PATH, ""))
                    if family == FAMILY_INET4:
                        addr.set_attrs({NETMASK_ATTR:
                                    old_net_dev.get_value(SUBNET_PATH, "")})
                    else:
                        addr.set_attrs({PREFIXLEN_ATTR:
                                    old_net_dev.get_value(PREFIXLEN_PATH, "")})

                protocollist.append_child(protocol)
            except:
                # Ignore error
                pass

        return netif

    def _update_netiflist(self, old_landiskinfo, new_landiskinfo):
        """Update "netiflist" of network setting in "new_landiskinfo"
           by setting in "old_landiskinfo".
        """

        netiflist = new_landiskinfo.get_node(NETIFLIST_PATH)

        for netif_name in sorted(NETIF_OLD_XPATH_MAP.keys()):
            NETIF_BASE_MAP_INFO_LIST = [
                ["name", NO_TAG_EXISTS_NAME, netif_name, False],
                ["netif_enable", "@enable", TRUE_TEXT, False],
                ["wol_enable", "wol/@enable", TRUE_TEXT, False],
                ["mtu", "frame/framesize", "1500", False],
            ]

            try:
                netiflist.append_child(self._get_netif(old_landiskinfo,
                                        NETIF_OLD_XPATH_MAP[netif_name],
                                        NETIF_BASE_MAP_INFO_LIST))
            except:
                # Ignore error
                pass

    def _transform_network(self, old_landiskinfo, new_landiskinfo):
        """Transform old network setting in "old_landiskinfo" into
           "new_landiskinfo".
        """

        self._update_old_tcpip(old_landiskinfo)
        self._update_familyconfig(old_landiskinfo, new_landiskinfo)
        self._update_netiflist(old_landiskinfo, new_landiskinfo)

    def _update_ftp(self, old_landiskinfo, new_landiskinfo):
        """Update a FTP setting in "new_landiskinfo"
           by setting in "old_landiskinfo".
        """

        ftp_encode = new_landiskinfo.get_node(FTP_ENCODE_PATH)
        if ftp_encode.get_text() in FTP_ENCODE_MAP:
            ftp_encode.set_text(FTP_ENCODE_MAP[ftp_encode.get_text()])
        for (new_xpath, old_xpath) in (
                    (FTP_CONTROL_NUMBER_XPATH, OLD_FTP_CONTROL_NUMBER_XPATH),
                    (FTP_TRANSFER_NUMBER_XPATH, OLD_FTP_TRANSFER_NUMBER_XPATH),
                ):

            try:
                old_port = old_landiskinfo.get_value(old_xpath)
                if int(old_port):
                    new_landiskinfo.get_node(new_xpath).set_text(old_port)
            except:
                # Ignore error
                pass

    def _transform_lockkey(self, old_landiskinfo, new_landiskinfo):
        """Transform old lockkey setting in "old_landiskinfo" into
           "new_landiskinfo".
        """

        old_server_list = old_landiskinfo.get_node_list(OLD_SERVER_PATH)
        if not len(old_server_list):
            return

        old_keyserver = old_landiskinfo.get_node(OLD_KEYSERVER_PATH)
        lockkey = XmlConfNode(LOCKKEY_BASE_XML)
        keyserverlist = lockkey.get_node(KEYSERVERLIST_NAME)
        for old_server in old_server_list:
            is_enable = FALSE_TEXT
            if old_keyserver.is_enabled() and old_server.get_value(HOST_NAME):
                is_enable = TRUE_TEXT

            try:
                new_no = str(int(old_server.get_value("@priorityorder")) + 1)
            except:
                # Skip old server entry.
                self.exception("can't get priorityorder for keyserver.")
                continue

            KEYSERVER_BASE_MAP_INFO_LIST = [
                    ["enable", NO_TAG_EXISTS_NAME, is_enable, False],
                    ["no", NO_TAG_EXISTS_NAME, new_no, False],
                    ["hostname", "host", "", False],
                    ["port", "port", "", False],
                ]

            keyserver = self._transform_xml_data(old_server,
                                        NEW_KEYSERVER_BASE_TMPL,
                                        KEYSERVER_BASE_MAP_INFO_LIST)
            keyserverlist.append_child(keyserver)
            
        new_landiskinfo.append_child(lockkey)

    def _update_msnetwork(self, old_landiskinfo, new_landiskinfo):
        """Update a MS Network setting in "new_landiskinfo"
           by setting in "old_landiskinfo".
        """

        assign = new_landiskinfo.get_node(ASSIGN_PATH)
        old_msnetwork = old_landiskinfo.get_node(OLD_MSNETWORK_PATH)
        for child_name in ASSIGN_CHILD_NAME_LIST:
            try:
                old_child = old_msnetwork.get_node(child_name)
                assign.append_child(old_child.get_clone())
            except:
                # Ignore error
                pass

    def _get_mailto_dict_map(self, old_landiskinfo):
        """Return a map of dictionary for "mailto" setting in
           "old_landiskinfo".
        """

        TOADDR_KEY = "toaddr"
        REPORT_KEY = "report_enable"
        NODE_NAME_MAP = {
                "poweron": ["system_enable"],
                "backup": ["backup_enable"],
                "temperature": ["temperature_enable"],
                "temparature": ["temperature_enable"],
                "syslog": ["eventlog_enable", "accesslog_enable"],
                "diskerror": ["disk_enable"],
                "diskfull": ["volume_enable"],
                "ups": ["ups_enable"],
            }

        DEFAULT_NOTICE_DICT = {
                "system_enable": FALSE_TEXT,
                "backup_enable": FALSE_TEXT,
                "eventlog_enable": FALSE_TEXT,
                "accesslog_enable": FALSE_TEXT,
                "temperature_enable": FALSE_TEXT,
                "disk_enable": FALSE_TEXT,
                "volume_enable": FALSE_TEXT,
                "quota_enable": FALSE_TEXT,
                "ups_enable": FALSE_TEXT,
                "report_enable": FALSE_TEXT,
            }

        mailto_dict_map = {}
        old_diskuse = old_landiskinfo.get_node(OLD_REPORT_DISKUSE_PATH)
        for mailto in old_diskuse.get_value_list(MAILTO_NAME):
            if not mailto or not old_diskuse.is_enabled():
                continue

            if mailto not in mailto_dict_map:
                mailto_dict_map[mailto] = {}
                mailto_dict_map[mailto].update(DEFAULT_NOTICE_DICT)
                mailto_dict_map[mailto][TOADDR_KEY] = mailto

            mailto_dict_map[mailto][REPORT_KEY] = TRUE_TEXT

        for child in old_landiskinfo.get_node_list(OLD_EVENTNOTICE_CHILD_PATH):
            notice_key_list = NODE_NAME_MAP.get(child.get_node_name(),[])
            if not notice_key_list:
                continue

            for mailto in child.get_value_list(MAILTO_NAME):
                if mailto not in mailto_dict_map:
                    mailto_dict_map[mailto] = {}
                    mailto_dict_map[mailto].update(DEFAULT_NOTICE_DICT)
                    mailto_dict_map[mailto][TOADDR_KEY] = mailto

                for notice_key in notice_key_list:
                    mailto_dict_map[mailto][notice_key] = TRUE_TEXT

        return mailto_dict_map

    def _update_mailnotice(self, old_landiskinfo, new_landiskinfo):
        """Update a mailnotice setting in "new_landiskinfo"
           by setting in "old_landiskinfo".
        """

        mailconflist = new_landiskinfo.get_node(MAILCONFLIST_PATH)
        mailbasic_mailconf = new_landiskinfo.get_node(MAILBASIC_MAILCONF_PATH)

        old_mail_detail_dict = {}
        for mailtoinfo in old_landiskinfo.get_node_list(OLD_MAILTOINFO_PATH):
            if (not mailtoinfo.get_value(MAILTO_NAME) or 
                not mailtoinfo.get_value(ENCODE_NAME)):

                continue

            old_mail_detail_dict[mailtoinfo.get_value(MAILTO_NAME)] = \
                                    mailtoinfo.get_value(ENCODE_NAME).lower()

        mailto_dict_map = self._get_mailto_dict_map(old_landiskinfo)
        count = 0
        for mailto in mailto_dict_map: 
            if count >= MAX_MAILTO_NUM:
                return

            mailconf = XmlConfNode(NEW_MAILCONF_BASE_TMPL %
                                        mailto_dict_map[mailto])
            
            for (old_mailto, encode) in old_mail_detail_dict.items():
                if (mailto == old_mailto and
                    encode != mailbasic_mailconf.get_value(ENCODE_NAME)):

                    mailconf.set_bool_attr(DETAIL_ATTR)
                    mailconf.update(mailbasic_mailconf)
                    mailconf.get_node(ENCODE_NAME).set_text(encode)

            mailconflist.append_child(mailconf)

    def _update_regularnotice(self, old_landiskinfo, new_landiskinfo):
        """Update a regularnotice setting in "new_landiskinfo"
           by setting in "old_landiskinfo".
        """

        regularnotice = new_landiskinfo.get_node(REGULARNOTICE_PATH)
        for child in regularnotice.get_node_list(ANY_CHILD_PATH):
            if child.is_enabled():
                regularnotice.enable()
                break

    def _update_report(self, old_landiskinfo, new_landiskinfo):
        """Update a report setting in "new_landiskinfo"
           by setting in "old_landiskinfo".
        """

        old_diskuse = old_landiskinfo.get_node(OLD_REPORT_DISKUSE_PATH)
        if not old_diskuse.is_enabled():
            return

        report = new_landiskinfo.get_node(REPORT_PATH)
        report.enable()

        eventtimelist = report.get_node(EVENTTIMELIST_NAME)
        for old_day in old_landiskinfo.get_node_list(OLD_REPORT_DAY_PATH):
            day = old_day.get_text()
            if not day:
                continue

            EVENTTIME_BASE_MAP_INFO_LIST = [
                    ["day", NO_TAG_EXISTS_NAME, day.capitalize(), False],
                    ["hour", "time/hour", "", False],
                    ["minute", "time/minute", "", False],
                ]

            eventtime = self._transform_xml_data(old_diskuse,
                                        NEW_EVENTTIME_BASE_TMPL,
                                        EVENTTIME_BASE_MAP_INFO_LIST)
            eventtimelist.append_child(eventtime)

    def _update_notice(self, old_landiskinfo, new_landiskinfo):
        """Update a notice setting in "new_landiskinfo"
           by setting in "old_landiskinfo".
        """

        self._update_mailnotice(old_landiskinfo, new_landiskinfo)
        self._update_regularnotice(old_landiskinfo, new_landiskinfo)
        self._update_report(old_landiskinfo, new_landiskinfo)

    def _update_portconfig(self, old_landiskinfo, new_landiskinfo):
        """Update a portconfig setting in "new_landiskinfo"
           by setting in "old_landiskinfo".
        """

        old_usb = old_landiskinfo.get_node(OLD_USB1_PATH)
        old_enable_mode = old_usb.get_attr_value(MODE_ATTR)
        quickcopy_dest = old_usb.get_value(QUICKCOPY_NAME)
        for mode in new_landiskinfo.get_node_list(USB1_MODE_PATH):
            if mode.get_text() != old_enable_mode:
                mode.enable(False)
                continue

            mode.enable()
            if old_enable_mode == QUICKCOPY_NAME and quickcopy_dest:
                mode.set_attrs({SHARE_ATTR: quickcopy_dest})

    def _transform_userlist(self, old_landiskinfo, new_landiskinfo):
        """Transform old user setting in "old_landiskinfo" into
           "new_landiskinfo".
        """

        USER_BASE_MAP_INFO_LIST = [
                ["name", "name", "", False],
                ["passwd", "passwd", "", False],
                ["fullname", "fullname", "", False],
                ["desc", "desc", "", False],
                ["quota_limit", "quota/limit", "0.0", False],
            ]

        userlist = new_landiskinfo.get_node(USERLIST_PATH)
        for old_user in old_landiskinfo.get_node_list(OLD_USER_PATH):

            user = self._transform_xml_data(old_user,
                                        NEW_USER_BASE_TMPL,
                                        USER_BASE_MAP_INFO_LIST)

            if not user.get_value(NAME_NAME):
                continue
            userlist.append_child(user)

    def _transform_grouplist(self, old_landiskinfo, new_landiskinfo):
        """Transform old group setting in "old_landiskinfo" into
           "new_landiskinfo".
        """

        GROUP_BASE_MAP_INFO_LIST = [
                ["name", "name", "", False],
                ["quota_limit", "quota/limit", "0.0", False],
            ]

        grouplist = new_landiskinfo.get_node(GROUPLIST_PATH)
        for old_group in old_landiskinfo.get_node_list(OLD_GROUP_PATH):

            group = self._transform_xml_data(old_group,
                                        NEW_GROUP_BASE_TMPL,
                                        GROUP_BASE_MAP_INFO_LIST)
            try:
                group.append_child(
                            old_group.get_node(USERLIST_NAME).get_clone())
            except:
                # Ignore error
                pass

            grouplist.append_child(group)

    def _transform_sharelist(self, old_landiskinfo, new_landiskinfo):
        """Transform old share setting in "old_landiskinfo" into
           "new_landiskinfo".
        """

        SHARE_BASE_MAP_INFO_LIST = [
                ["name", "name", "", False],
                ["path", "sharetarget", "", False],
                ["desc", "desc", "", False],
                ["readonly_enable", "permission/readonly/@enable",
                    FALSE_TEXT, False],
                ["msshare_enable", "shareservice/msshare/@enable",
                    FALSE_TEXT, False],
                ["trash_enable", "shareservice/msshare/trash/@enable",
                    FALSE_TEXT, False],
                ["hidden_enable", "shareservice/msshare/hidden/@enable",
                    FALSE_TEXT, False],
                ["quota_limit", "shareservice/msshare/quota/limit",
                    "0.0", False],
                ["noguest_enable", "shareservice/noguest/@enable",
                    FALSE_TEXT, False],
                ["ftp_enable", "shareservice/ftp/@enable",
                    FALSE_TEXT, False],
                ["appleshare_enable", "shareservice/appleshare/@enable",
                    FALSE_TEXT, False],
                ["timemachine_enable",
                    "shareservice/appleshare/timemachine/@enable",
                    FALSE_TEXT, False],
            ]

        sharelist = new_landiskinfo.get_node(SHARELIST_PATH)
        for folder in old_landiskinfo.get_node_list(OLD_FOLDER_PATH):

            share = self._transform_xml_data(folder,
                                        NEW_SHARE_BASE_TMPL,
                                        SHARE_BASE_MAP_INFO_LIST)
            share.get_node(VOLUME_NAME).set_text(
                        self._get_volume(folder.get_value(SHARETARGET_NAME)))
            try:
                share.get_node(PERMISSION_NAME).append_child(
                            folder.get_node(DETAILPERMISSION_PATH).get_clone())
            except:
                # Ignore error
                pass

            sharelist.append_child(share)

    def _transform_landiskinfo(self, landiskinfo):
        """Transform "landiskinfo" for HDL-H series."""

        UPS_TARGETLANDISK_PATH = \
                "system/ups/netshutdown/targetlandisklist/targetlandisk"

        LANDISKINFO_BASE_MAP_INFO_LIST = [
                ["update", "@update", "", False],
                ["version", "@version", "", False],
                ["product_name", "product/name", "", False],
                ["firmware", "product/firmware", "", False],
                ["applenetwork_enable",
                    "network/shareservice/appleshare/@enable",
                    FALSE_TEXT, False],
                ["eco_time", "system/eco/defaulttime/minute", "0", False],
                ["eco_sataraid1_enable",
                    "system/eco/internal/@enable", FALSE_TEXT, False],
                ["eco_usb1_enable",
                    """system/eco/usb[@port="1"]/@enable""",
                    FALSE_TEXT, False],
                ["eco_usb2_enable",
                    """system/eco/usb[@port="2"]/@enable""",
                    FALSE_TEXT, False],
                ["fw_updatenotice_enable",
                    "system/firmwareupdate/@enable", TRUE_TEXT, False],
                ["ftp_enable",
                    "network/shareservice/ftp/@enable", FALSE_TEXT, False],
                ["ftp_encode", """network/ftp/encode""", "", False],
                ["led", """system/led""", "", False],
                ["msnetwork_enable",
                    "network/shareservice/msshare/@enable", TRUE_TEXT, False],
                ["msnetwork_method", "network/msnetwork/@method", "", True],
                ["winssrv", "network/msnetwork/winssrv", "", False],
                ["mailconf_host", "system/mail/smtp/srv", "", False],
                ["mailconf_port", "system/mail/smtp/port", "", False],
                ["mailconf_fromaddr", "system/mail/mailfrom", "", False],
                ["mailconf_auth_type", "system/mail/mailauth/@type", "", True],
                ["mailconf_auth_user",
                    "system/mail/mailauth/senduser/name", "", False],
                ["mailconf_auth_password",
                    "system/mail/mailauth/senduser/passwd", "", False],
                ["mailconf_pop_host",
                    "system/mail/mailauth/popsrv", "", False],
                ["mailconf_encode", "system/mail/encode", "", True],
                ["notice_diskerror_enable",
                    "system/eventnotice/diskerror/@enable", FALSE_TEXT, False],
                ["notice_ups_enable",
                    "system/eventnotice/ups/@enable", FALSE_TEXT, False],
                ["notice_volume_enable",
                    "system/eventnotice/diskfull/@enable", FALSE_TEXT, False],
                ["notice_volume_value",
                    "system/eventnotice/diskfull/warningvalue", "", False],
                ["notice_quota_enable",
                    "system/eventnotice/quota/@enable", FALSE_TEXT, False],
                ["notice_quota_value",
                    "system/eventnotice/quota/warningvalue", "", False],
                ["autoresume_enable",
                    "system/powersetting/autoresume/@enable",
                    FALSE_TEXT, False],
                ["quota_enable",
                    "network/shareservice/quota/@enable", FALSE_TEXT, False],
                ["activerepair_enable",
                    "volume/activerepair/@enable", TRUE_TEXT, False],
                ["activerepair_day",
                    "volume/activerepair/day", "Sat", False],
                ["activerepair_hour",
                    "volume/activerepair/time/hour", "0", False],
                ["activerepair_minute",
                    "volume/activerepair/time/minute", "0", False],
                ["reset_enable",
                    "system/switch/resetbutton/@enable", TRUE_TEXT, False],
                ["func_enable",
                    "system/switch/funcbutton/@enable", TRUE_TEXT, False],
                ["host_name", "network/host/name", "", False],
                ["host_desc", "network/host/desc", "", False],
                ["accesslog_enable",
                    "network/shareservice/msshare/accesslog/@enable",
                    FALSE_TEXT, False],
                ["ups_target1_enable",
                    UPS_TARGETLANDISK_PATH + """[@number="1"]/@enable""",
                    FALSE_TEXT, False],
                ["ups_target1_host",
                    UPS_TARGETLANDISK_PATH + """[@number="1"]/host""",
                    "", False],
                ["ups_target1_passwd",
                    UPS_TARGETLANDISK_PATH + """[@number="1"]/passwd""",
                    "", False],
                ["ups_target2_enable",
                    UPS_TARGETLANDISK_PATH + """[@number="2"]/@enable""",
                    FALSE_TEXT, False],
                ["ups_target2_host",
                    UPS_TARGETLANDISK_PATH + """[@number="2"]/host""",
                    "", False],
                ["ups_target2_passwd",
                    UPS_TARGETLANDISK_PATH + """[@number="2"]/passwd""",
                    "", False],
                ["ups_target3_enable",
                    UPS_TARGETLANDISK_PATH + """[@number="3"]/@enable""",
                    FALSE_TEXT, False],
                ["ups_target3_host",
                    UPS_TARGETLANDISK_PATH + """[@number="3"]/host""",
                    "", False],
                ["ups_target3_passwd",
                    UPS_TARGETLANDISK_PATH + """[@number="3"]/passwd""",
                    "", False],
                ["ups_target4_enable",
                    UPS_TARGETLANDISK_PATH + """[@number="4"]/@enable""",
                    FALSE_TEXT, False],
                ["ups_target4_host",
                    UPS_TARGETLANDISK_PATH + """[@number="4"]/host""",
                    "", False],
                ["ups_target4_passwd",
                    UPS_TARGETLANDISK_PATH + """[@number="4"]/passwd""",
                    "", False],
                ["ups_target5_enable",
                    UPS_TARGETLANDISK_PATH + """[@number="5"]/@enable""",
                    FALSE_TEXT, False],
                ["ups_target5_host",
                    UPS_TARGETLANDISK_PATH + """[@number="5"]/host""",
                    "", False],
                ["ups_target5_passwd",
                    UPS_TARGETLANDISK_PATH + """[@number="5"]/passwd""",
                    "", False],
                ["ups_warning_enable",
                    "system/ups/warning/@enable", FALSE_TEXT, False],
                ["ups_time", "system/ups/time/minute", "10", False],
            ]

        new_landiskinfo = self._transform_xml_data(landiskinfo,
                                        NEW_LANDISKINFO_BASE_TMPL,
                                        LANDISKINFO_BASE_MAP_INFO_LIST)

        try:
            self._transform_network(landiskinfo, new_landiskinfo)
        except:
            self.exception("Transform network")
            # Ignore error
            pass

        try:
            self._update_ftp(landiskinfo, new_landiskinfo)
        except:
            self.exception("Update ftp")
            # Ignore error
            pass

        try:
            self._transform_lockkey(landiskinfo, new_landiskinfo)
        except:
            self.exception("Transform lockkey")
            # Ignore error
            pass

        try:
            self._update_msnetwork(landiskinfo, new_landiskinfo)
        except:
            self.exception("Update MS Network")
            # Ignore error
            pass

        try:
            self._update_notice(landiskinfo, new_landiskinfo)
        except:
            self.exception("Update notice")
            # Ignore error
            pass

        try:
            self._update_portconfig(landiskinfo, new_landiskinfo)
        except:
            self.exception("Update portconfig")
            # Ignore error
            pass

        try:
            self._transform_userlist(landiskinfo, new_landiskinfo)
        except:
            self.exception("Transform userlist")
            # Ignore error
            pass

        try:
            self._transform_grouplist(landiskinfo, new_landiskinfo)
        except:
            self.exception("Transform grouplist")
            # Ignore error
            pass

        try:
            self._transform_sharelist(landiskinfo, new_landiskinfo)
        except:
            self.exception("Transform sharelist")
            # Ignore error
            pass

        try:
            new_landiskinfo.get_node(POWERSETTING_PATH).append_child(
                landiskinfo.get_node(OLD_POWERSCHEDULE_PATH).get_clone())
        except:
            self.exception("Transform powerschedule")

            # Ignore error
            pass

        COPY_NODE_LIST = ("datetime", "proxy", "snmp")

        for name in COPY_NODE_LIST:
            try:
                new_landiskinfo.append_child(
                    landiskinfo.get_node(name).get_clone())
            except:
                # Ignore error
                pass

        return new_landiskinfo

    def _get_remote_landiskinfo(self):
        """Return the landiskinfo XML data of landisk set in config file."""

        config = util.load_config(CONFIG_FILE)
        source = config.get_node(SOURCE_PATH).get_clone()
        get_entry = Entry(ReqTypes.GET, MIGRATION_NAME)
        get_entry.get().append_child(source)
        response_entry = self.send_entry(get_entry,
                                pal_info.PAL_MIGRATION_PATH)
        if response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

        landiskinfo = response_entry.get().get_node(LANDISKINFO_NAME)
        try:
            landiskinfo.delete_children(RESTOREUI_NAME)
        except:
            # ignore error
            pass

        if self._get_landiskinfo_version(landiskinfo) < HDL_H_MIN_VER:
            landiskinfo = self._transform_landiskinfo(landiskinfo)

        return landiskinfo

    def _save_landiskinfo(self, landiskinfo):
        """Save "landiskinfo" XML data to file."""

        if not os.path.isdir(os.path.dirname(LANDISK_INFO_XML_PATH)):
            os.makedirs(os.path.dirname(LANDISK_INFO_XML_PATH))

        tmp_path = LANDISK_INFO_XML_PATH + "." + str(os.getpid())
        XmlConf(landiskinfo).save(tmp_path)
        os.rename(tmp_path, LANDISK_INFO_XML_PATH)

    def _is_share_ignored(self, share):
        """Return if "share" should be ignored."""

        try:
            if share.is_set_to_true(UNDELETABLE_ATTR):
                return True

            name = share.get_node(NAME_NAME).get_text()
            if not name or name in IGNORED_SHARE_LIST:
                return True
        except:
            # Assume target to be ignored.
            return True

        return False

    def _delete_ignored_and_network_setting(self, landiskinfo):
        """Delete ignored and network setting in "landiskinfo"."""

        IGNORED_SETTING_PATH_PAIR_LIST = [
                (ROOT_PATH, "raid"),
                (ROOT_PATH, "network"),
                (ROOT_PATH, "network.v2"),
                ("msnetwork", "assign"),
                ("msnetwork", "winssrv"),
                (ROOT_PATH, "servicectrl"),
                (ROOT_PATH, "sysinfo"),
                (ROOT_PATH, "volctrl"),
            ]

        for (parent_path, child_name) in IGNORED_SETTING_PATH_PAIR_LIST:
            landiskinfo.delete_nodes(parent_path, child_name)

        idx = 0
        sharelist = landiskinfo.get_node(SHARELIST_PATH)
        for share in sharelist.get_node_list(SHARE_NAME):
            if self._is_share_ignored(share):
                sharelist.delete_child(SHARE_NAME, index=idx)
            else:
                idx += 1

        idx = 0
        usblist = landiskinfo.get_node(USBLIST_PATH)
        for usb in usblist.get_node_list(USB_NAME):
            if usb.get_attr_value(PORT_ATTR) != PORTCONFIG_TARGET_PORT:
                usblist.delete_child(USB_NAME, index=idx)
            else:
                idx += 1

    def _delete_non_network_setting(self, landiskinfo):
        """Delete non network setting in "landiskinfo"."""

        NON_NETWORK_SETTING_PATH_PAIR_LIST = [
                (ROOT_PATH, "raid"),
                (ROOT_PATH, "applenetwork"),
                (ROOT_PATH, "backups"),
                (ROOT_PATH, "datetime"),
                (ROOT_PATH, "eco"),
                (ROOT_PATH, "eventlog"),
                (ROOT_PATH, "firmware"),
                (ROOT_PATH, "ftp"),
                (ROOT_PATH, "groups"),
                (ROOT_PATH, "led"),
                (ROOT_PATH, "lockkey"),
                (ROOT_PATH, "notice"),
                (ROOT_PATH, "packages"),
                (ROOT_PATH, "portconfig"),
                (ROOT_PATH, "powersetting"),
                (ROOT_PATH, "proxy"),
                (ROOT_PATH, "quota"),
                (ROOT_PATH, "raidctrl"),
                (ROOT_PATH, "servicectrl"),
                (ROOT_PATH, "shares"),
                (ROOT_PATH, "snmp"),
                (ROOT_PATH, "switch"),
                (ROOT_PATH, "systemlog"),
                (ROOT_PATH, "ups"),
                (ROOT_PATH, "users"),
                (ROOT_PATH, "volctrl"),
            ]

        for (parent_path, child_name) in NON_NETWORK_SETTING_PATH_PAIR_LIST:
            landiskinfo.delete_nodes(parent_path, child_name)

    def _disable_replication_and_schedule_setting(self, landiskinfo):
        """Disable replication and schedule setting."""

        DISABLE_NODE_XPATH = [
                "backups/backup/joblist/job/schedule",
                "backups/datacopy/joblist/job/schedule",
                "shares/sharelist/share/replication",
                "powersetting/powerschedule/dayschedule/offtime",
            ]

        for xpath in DISABLE_NODE_XPATH:
            for node in landiskinfo.get_node_list(xpath):
                node.enable(False)

    def _get_network_setting(self, landiskinfo):
        """Return a landiskinfo XML data for setting about network.
        """

        new_landiskinfo = XmlConfNode(LANDISKINFO_NAME)

        return new_landiskinfo

    def _execute_migration(self, landiskinfo):
        """Execute migration for "landiskinfo"."""

        config = util.load_config(CONFIG_FILE)
        execute_entry = Entry(ReqTypes.EXECUTE, MIGRATION_NAME)
        execute_entry.get().append_child(landiskinfo)
        job = config.get_node(JOB_NAME)
        execute_entry.get().append_child(job)
        response_entry = self.send_entry(execute_entry,
                                pal_info.PAL_MIGRATION_PATH)
        if response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

    def _get_fixed_ip_address(self, landiskinfo):
        """Return a list of fixed IP address in "landiskinfo"."""

        ip_address_list = []
        for protocol in landiskinfo.get_node_list(LANDISKINFO_PROTOCOL_PATH):
            try:
                if protocol.get_node(DHCP_NAME).is_enabled():
                    continue
                ip_address = protocol.get_value(ADDR_PATH)
                if ip_address:
                    ip_address_list.append(ip_address)
            except:
                # Ignore error
                pass

        return ip_address_list

    def _switch_network_setting(self, landiskinfo):
        """Switch network setting for "landiskinfo"."""

        switch_entry = Entry(ReqTypes.SWITCH, MIGRATION_NAME)
        switch_entry.get().append_child(landiskinfo)
        response_entry = self.send_entry(switch_entry,
                                pal_info.PAL_MIGRATION_PATH)
        if response_entry.get_result() != Results.OK:
            raise RequestError(response_entry.get_result())

    #
    # Request handlers
    #

    def _list_handler(self, entry):
        """Handle list request."""

        self._check_body_node(entry.get())
        return self.send_entry(entry, pal_info.PAL_MIGRATION_PATH)

    def _query_handler(self, entry):
        """Handle query request."""

        self._check_body_node(entry.get())
        response_entry = self.send_entry(entry, pal_info.PAL_MIGRATION_PATH)
        if response_entry.get_result() != Results.OK:
            return response_entry

        try:
            landiskinfo = XmlConf(LANDISK_INFO_XML_PATH).get_root_node()
            statusinfo = response_entry.get().get_node(STATUSINFO_PATH)
            host_name = landiskinfo.get_value(LANDISKINFO_HOST_NAME_PATH)
            product_name = landiskinfo.get_value(LANDISKINFO_PRODUCT_NAME_PATH)
            statusinfo.append_child(XmlConfNode(HOST_XML_TMPL % (host_name,)))
            statusinfo.append_child(
                            XmlConfNode(PRODUCT_XML_TMPL % (product_name,)))
        except:
            # Ignore error
            pass
        response_entry.set_result(Results.OK)
        return response_entry

    def _apply_handler(self, entry):
        """Handle apply request."""

        self._check_body_node(entry.get(), ReqTypes.APPLY)
        try:
            self._check_body_node(entry.get())
            response_entry = self.send_entry(entry,
                                                pal_info.PAL_MIGRATION_PATH)
            if response_entry.get_result() == Results.AUTH_ERROR:
                entry.set_result(Results.AUTH_ERROR)
            elif response_entry.get_result() != Results.OK:
                raise RequestError(response_entry.get_result())
            else:
                util.save_config(CONFIG_FILE, entry.get())
                entry.set_result(Results.OK)
        except:
            self.exception("apply fail")
            entry.set_result(Results.APPLY_ERROR)

        return entry

    def _execute_handler(self, entry):
        """Handle execute request."""

        self._check_body_node(entry.get())
        try:
            landiskinfo = self._get_remote_landiskinfo()
            self._save_landiskinfo(landiskinfo)

            self._delete_ignored_and_network_setting(landiskinfo)
            self._disable_replication_and_schedule_setting(landiskinfo)
            self._execute_migration(landiskinfo)
            entry.set_result(Results.OK)
        except RequestError, err:
            if err.get_info() == Results.AUTH_ERROR:
                self.exception("execute auth error")
                entry.set_result(Results.AUTH_ERROR)
            elif err.get_info() == Results.CONNECT_ERROR:
                self.exception("execute connect error")
                entry.set_result(Results.CONNECT_ERROR)
            else:
                self.exception("execute request error")
                entry.set_result(Results.EXECUTE_ERROR)
        except:
            self.exception("execute error")
            entry.set_result(Results.EXECUTE_ERROR)
        return entry

    def _check_handler(self, entry):
        """Handle check request."""

        landiskinfo = XmlConf(LANDISK_INFO_XML_PATH).get_root_node()
        host_name = landiskinfo.get_value(LANDISKINFO_HOST_NAME_PATH)

        check_entry = Entry(ReqTypes.CHECK, MIGRATION_NAME)
        check_entry.get().append_child(HOSTNAMELIST_NAME)
        hostnamelist = check_entry.get().get_node(HOSTNAMELIST_NAME)

        for hostname in [host_name] + self._get_fixed_ip_address(landiskinfo):
            hostnamelist.append_child(HOSTNAME_NAME, hostname)

        response_entry = self.send_entry(check_entry,
                                            pal_info.PAL_MIGRATION_PATH)
        entry.set_result(response_entry.get_result())
        return entry

    def _switch_handler(self, entry):
        """Handle switch request."""

        self._check_body_node(entry.get())
        try:
            landiskinfo = XmlConf(LANDISK_INFO_XML_PATH).get_root_node()
            self._delete_non_network_setting(landiskinfo)
            self._switch_network_setting(landiskinfo)
            entry.set_result(Results.OK)
        except:
            self.exception("switch error")
            entry.set_result(Results.EXECUTE_ERROR)

        return entry

    #
    # Constructor
    #

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_save_handler = lambda entry: self.send_entry_with_file(
            entry, pal_info.PAL_CONFFILE_PATH, CONFIG_FILE)

        self.set_handler(ReqTypes.LOAD, load_save_handler)
        self.set_handler(ReqTypes.SAVE, load_save_handler)

        self.set_handler(ReqTypes.LIST, self._list_handler)
        self.set_handler(ReqTypes.QUERY, self._query_handler)
        self.set_handler(ReqTypes.APPLY, self._apply_handler)
        self.set_handler(ReqTypes.EXECUTE, self._execute_handler)
        self.set_handler(ReqTypes.CHECK, self._check_handler)
        self.set_handler(ReqTypes.SWITCH, self._switch_handler)


#
# Main
#

if __name__ == "__main__":
    FalMigration(option=falcmd.FalCmdOption()).handle_request()
